@echo off
call check-for-admin
if %ERRORLEVEL% neq 0 exit /b

set OPT=..\optional
set CONFIG=config
set SOFTWARE=..\software
set MSI=softmaker-setup-2024.msi 
set SETTINGS=..\company\settings.cmd
set softmaker_license=0
set STARTMENU="%PROGRAMDATA%\Microsoft\Windows\Start Menu\Programs"

if NOT EXIST %SOFTWARE%\%MSI% (
  echo ERROR: %SOFTWARE%\%MSI% not found!
  exit /b
)

rem ===================
rem CONFIGFILE CHECK
rem ===================

if NOT EXIST %SETTINGS% (
  echo.
  echo WARNING: %SETTINGS% NOT FOUND
  echo.
) else (
  call %SETTINGS%
)

echo ####### %0 #######

cd %SOFTWARE%
@echo on
msiexec /i %MSI% /qb APPLICATIONFOLDER="c:\Program Files\Softmaker Office 2024" ^
  INSTALLTM=1 INSTALLPM=1 INSTALLPR=1 INSTALLTB1=0 INSTALLTB2=0 /l*v c:\temp\softmaker.log
@echo off
del /F c:\temp\softmaker.log
cd %OPT%

echo installing desktop icons ...
copy /Y "%PROGRAMDATA%\Microsoft\Windows\Start Menu\Programs\Softmaker Office 2024\PlanMaker 2024.lnk" ^
  %PUBLIC%\Desktop\PlanMaker.lnk
copy /Y "%PROGRAMDATA%\Microsoft\Windows\Start Menu\Programs\Softmaker Office 2024\Presentations 2024.lnk" ^
  %PUBLIC%\Desktop\Presentations.lnk
copy /Y "%PROGRAMDATA%\Microsoft\Windows\Start Menu\Programs\Softmaker Office 2024\TextMaker 2024.lnk" ^
  %PUBLIC%\Desktop\Textmaker.lnk

rem refresh desktop (W10 style)
ie4uinit.exe -show

echo cleaning startmenu ...
del /F /Q /A %STARTMENU%\"Softmaker Office 2024\Bitte*.*" 2>nul

echo disabling automatic updates for Softmaker Office 2024 ...
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\SoftMaker Office 2024" ^
  /v "UpdateCheckEnabled" /d 0 /t REG_DWORD /f

echo disabling WEB help ...
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\SoftMaker Office 2024" ^
  /v "HelpSoftMaker" /d 0 /t REG_DWORD /f

echo disabling License Management ...
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\SoftMaker Office 2024" ^
  /v "HelpManage" /d 0 /t REG_DWORD /f

echo disabling the asking for user infos ...
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\SoftMaker Office 2024" ^
  /v "AskUserInfo" /d 0 /t REG_DWORD /f

echo setting dialog language to US-english ...
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\SoftMaker Office 2024" ^
  /v "DlgLID" /d 409 /t REG_DWORD /f

echo disabling backups ...
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\TextMaker 2024" ^
  /v "AutoSave" /d 0 /t REG_DWORD /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\PlanMaker 2024" ^
  /v "AutoSave" /d 0 /t REG_DWORD /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\Presentations 2024" ^
  /v "AutoSave" /d 0 /t REG_DWORD /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\TextMaker 2024" ^
  /v "BackupFilesNumber" /d 1 /t REG_DWORD /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\PlanMaker 2024" ^
  /v "BackupFilesNumber" /d 1 /t REG_DWORD /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\Presentations 2024" ^
  /v "BackupFilesNumber" /d 1 /t REG_DWORD /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\TextMaker 2024" ^
  /v "BackupFilesPath" /d "" /t REG_SZ /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\PlanMaker 2024" ^
  /v "BackupFilesPath" /d "" /t REG_SZ /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\Presentations 2024" ^
  /v "BackupFilesPath" /d "" /t REG_SZ /f

echo copying .ini files ...
mkdir %USERPROFILE%\Documents\SoftMaker\Settings 2>nul
copy /Y %CONFIG%\*24config.ini %USERPROFILE%\Documents\SoftMaker\Settings

echo preparing Softmaker documents folders ...
for %%D in (

  "Backup"
  "Bullets"
  "Samples"
  "PlanMaker 2024 templates"
  "Presentations 2024 designs"
  "TextMaker 2024 templates"

) do (

  mkdir %USERPROFILE%\Documents\SoftMaker\%%D 2>/nul
  attrib +R %USERPROFILE%\Documents\SoftMaker\%%D 2>/nul

)

echo setting default formats ...
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\TextMaker 2024" ^
  /v "DefaultFormat" /d 6 /t REG_DWORD /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\PlanMaker 2024" ^
  /v "DefaultFormat" /d 6 /t REG_DWORD /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\Presentations 2024" ^
  /v "DefaultFormat" /d 6 /t REG_DWORD /f

echo disabling smart quotes ...
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\TextMaker 2024" ^
  /v "SmartQuotes" /d 0 /t REG_DWORD /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\PlanMaker 2024" ^
  /v "SmartQuotes" /d 0 /t REG_DWORD /f
reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\Presentations 2024" ^
  /v "SmartQuotes" /d 0 /t REG_DWORD /f

if softmaker_license == 0 (
  echo no softmaker license found!
) else (
  echo adding softmaker license ...
  reg add "HKEY_LOCAL_MACHINE\SOFTWARE\Policies\SoftMaker\SoftMaker Office 2024" ^
    /v "ProductKey" /d "%softmaker_license%" /t REG_SZ /f
)

echo ####### %0 #######

rem set filetypes ...
if EXIST ..\scripts\set-filetypes.cmd (
  call ..\scripts\set-filetypes.cmd
)

pause

